@@ -24,26 +24,44 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <math.h>
+#include <string.h>
+
 #include "gtkcontainer.h"
 #include "gtkimage.h"
 #include "gtkiconfactory.h"
 #include "gtkstock.h"
+#include "gtkicontheme.h"
 #include "gtkintl.h"
-#include <string.h>
+#include "gtkprivate.h"
+#include "gtkalias.h"
+
+typedef struct _GtkImagePrivate GtkImagePrivate;
+
+struct _GtkImagePrivate
+{
+  /* Only used with GTK_IMAGE_ANIMATION, GTK_IMAGE_PIXBUF */
+  gchar *filename;
+
+  gint pixel_size;
+};
+
+#define GTK_IMAGE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_IMAGE, GtkImagePrivate))
+
 
 #define DEFAULT_ICON_SIZE GTK_ICON_SIZE_BUTTON
-
-static void gtk_image_class_init   (GtkImageClass  *klass);
-static void gtk_image_init         (GtkImage       *image);
 static gint gtk_image_expose       (GtkWidget      *widget,
                                     GdkEventExpose *event);
 static void gtk_image_unmap        (GtkWidget      *widget);
 static void gtk_image_unrealize    (GtkWidget      *widget);
 static void gtk_image_size_request (GtkWidget      *widget,
                                     GtkRequisition *requisition);
+static void gtk_image_style_set    (GtkWidget      *widget,
+				    GtkStyle       *prev_style);
+static void gtk_image_screen_changed (GtkWidget    *widget,
+				      GdkScreen    *prev_screen);
 static void gtk_image_destroy      (GtkObject      *object);
-static void gtk_image_clear        (GtkImage       *image);
 static void gtk_image_reset        (GtkImage       *image);
 static void gtk_image_calc_size    (GtkImage       *image);
 
@@ -60,7 +78,7 @@ static void gtk_image_get_property      (GObject          *object,
 					 GValue           *value,
 					 GParamSpec       *pspec);
 
-static gpointer parent_class;
+static void icon_theme_changed          (GtkImage         *image);
 
 enum
 {
@@ -73,36 +91,13 @@ enum
   PROP_STOCK,
   PROP_ICON_SET,
   PROP_ICON_SIZE,
+  PROP_PIXEL_SIZE,
   PROP_PIXBUF_ANIMATION,
+  PROP_ICON_NAME,
   PROP_STORAGE_TYPE
 };
 
-GType
-gtk_image_get_type (void)
-{
-  static GType image_type = 0;
-
-  if (!image_type)
-    {
-      static const GTypeInfo image_info =
-      {
-	sizeof (GtkImageClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_image_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkImage),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_image_init,
-      };
-
-      image_type = g_type_register_static (GTK_TYPE_MISC, "GtkImage",
-					   &image_info, 0);
-    }
-
-  return image_type;
-}
+G_DEFINE_TYPE (GtkImage, gtk_image, GTK_TYPE_MISC)
 
 static void
 gtk_image_class_init (GtkImageClass *class)
@@ -111,8 +106,6 @@ gtk_image_class_init (GtkImageClass *class)
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class = G_OBJECT_CLASS (class);
   
   gobject_class->set_property = gtk_image_set_property;
@@ -128,99 +121,142 @@ gtk_image_class_init (GtkImageClass *class)
   widget_class->size_request = gtk_image_size_request;
   widget_class->unmap = gtk_image_unmap;
   widget_class->unrealize = gtk_image_unrealize;
+  widget_class->style_set = gtk_image_style_set;
+  widget_class->screen_changed = gtk_image_screen_changed;
   
   g_object_class_install_property (gobject_class,
                                    PROP_PIXBUF,
                                    g_param_spec_object ("pixbuf",
-                                                        _("Pixbuf"),
-                                                        _("A GdkPixbuf to display"),
+                                                        P_("Pixbuf"),
+                                                        P_("A GdkPixbuf to display"),
                                                         GDK_TYPE_PIXBUF,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_PIXMAP,
                                    g_param_spec_object ("pixmap",
-                                                        _("Pixmap"),
-                                                        _("A GdkPixmap to display"),
+                                                        P_("Pixmap"),
+                                                        P_("A GdkPixmap to display"),
                                                         GDK_TYPE_PIXMAP,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGE,
                                    g_param_spec_object ("image",
-                                                        _("Image"),
-                                                        _("A GdkImage to display"),
+                                                        P_("Image"),
+                                                        P_("A GdkImage to display"),
                                                         GDK_TYPE_IMAGE,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MASK,
                                    g_param_spec_object ("mask",
-                                                        _("Mask"),
-                                                        _("Mask bitmap to use with GdkImage or GdkPixmap"),
+                                                        P_("Mask"),
+                                                        P_("Mask bitmap to use with GdkImage or GdkPixmap"),
                                                         GDK_TYPE_PIXMAP,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_FILE,
                                    g_param_spec_string ("file",
-                                                        _("Filename"),
-                                                        _("Filename to load and display"),
+                                                        P_("Filename"),
+                                                        P_("Filename to load and display"),
                                                         NULL,
-                                                        G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
   
 
   g_object_class_install_property (gobject_class,
                                    PROP_STOCK,
                                    g_param_spec_string ("stock",
-                                                        _("Stock ID"),
-                                                        _("Stock ID for a stock image to display"),
+                                                        P_("Stock ID"),
+                                                        P_("Stock ID for a stock image to display"),
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SET,
-                                   g_param_spec_boxed ("icon_set",
-                                                       _("Icon set"),
-                                                       _("Icon set to display"),
+                                   g_param_spec_boxed ("icon-set",
+                                                       P_("Icon set"),
+                                                       P_("Icon set to display"),
                                                        GTK_TYPE_ICON_SET,
-                                                       G_PARAM_READWRITE));
+                                                       GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SIZE,
-                                   g_param_spec_int ("icon_size",
-                                                     _("Icon size"),
-                                                     _("Size to use for stock icon or icon set"),
+                                   g_param_spec_int ("icon-size",
+                                                     P_("Icon size"),
+                                                     P_("Symbolic size to use for stock icon, icon set or named icon"),
                                                      0, G_MAXINT,
                                                      DEFAULT_ICON_SIZE,
-                                                     G_PARAM_READWRITE));
-
+                                                     GTK_PARAM_READWRITE));
+  /**
+   * GtkImage:pixel-size:
+   *
+   * The :pixel-size property can be used to specify a fixed size
+   * overriding the :icon-size property for images of type 
+   * %GTK_IMAGE_ICON_NAME. 
+   *
+   * Since: 2.6
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_PIXEL_SIZE,
+				   g_param_spec_int ("pixel-size",
+						     P_("Pixel size"),
+						     P_("Pixel size to use for named icon"),
+						     -1, G_MAXINT,
+						     -1,
+						     GTK_PARAM_READWRITE));
+  
   g_object_class_install_property (gobject_class,
                                    PROP_PIXBUF_ANIMATION,
-                                   g_param_spec_object ("pixbuf_animation",
-                                                        _("Animation"),
-                                                        _("GdkPixbufAnimation to display"),
+                                   g_param_spec_object ("pixbuf-animation",
+                                                        P_("Animation"),
+                                                        P_("GdkPixbufAnimation to display"),
                                                         GDK_TYPE_PIXBUF_ANIMATION,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
+
+  /**
+   * GtkImage:icon-name:
+   *
+   * The name of the icon in the icon theme.  If the icon theme is
+   * changed, the image will be updated automatically.
+   *
+   * Since: 2.6
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_ICON_NAME,
+                                   g_param_spec_string ("icon-name",
+                                                        P_("Icon Name"),
+                                                        P_("The name of the icon from the icon theme"),
+                                                        NULL,
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_STORAGE_TYPE,
-                                   g_param_spec_enum ("storage_type",
-                                                      _("Storage type"),
-                                                      _("The representation being used for image data"),
+                                   g_param_spec_enum ("storage-type",
+                                                      P_("Storage type"),
+                                                      P_("The representation being used for image data"),
                                                       GTK_TYPE_IMAGE_TYPE,
                                                       GTK_IMAGE_EMPTY,
-                                                      G_PARAM_READABLE));
+                                                      GTK_PARAM_READABLE));
+
+  g_type_class_add_private (object_class, sizeof (GtkImagePrivate));
 }
 
 static void
 gtk_image_init (GtkImage *image)
 {
+  GtkImagePrivate *priv = GTK_IMAGE_GET_PRIVATE (image);
+
   GTK_WIDGET_SET_FLAGS (image, GTK_NO_WINDOW);
 
   image->storage_type = GTK_IMAGE_EMPTY;
   image->icon_size = DEFAULT_ICON_SIZE;
   image->mask = NULL;
+
+  priv->pixel_size = -1;
+
+  priv->filename = NULL;
 }
 
 static void
@@ -228,9 +264,9 @@ gtk_image_destroy (GtkObject *object)
 {
   GtkImage *image = GTK_IMAGE (object);
 
-  gtk_image_clear (image);
+  gtk_image_reset (image);
   
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_image_parent_class)->destroy (object);
 }
 
 static void 
@@ -277,14 +313,13 @@ gtk_image_set_property (GObject      *object,
           if (mask)
             g_object_ref (mask);
           
-          gtk_image_reset (image);
+          gtk_image_clear (image);
 
           image->mask = mask;
         }
       break;
     case PROP_FILE:
-      gtk_image_set_from_file (image,
-                               g_value_get_string (value));
+      gtk_image_set_from_file (image, g_value_get_string (value));
       break;
     case PROP_STOCK:
       gtk_image_set_from_stock (image, g_value_get_string (value),
@@ -303,15 +338,26 @@ gtk_image_set_property (GObject      *object,
         gtk_image_set_from_icon_set (image,
                                      image->data.icon_set.icon_set,
                                      g_value_get_int (value));
+      else if (image->storage_type == GTK_IMAGE_ICON_NAME)
+        gtk_image_set_from_icon_name (image,
+				      image->data.name.icon_name,
+				      g_value_get_int (value));
       else
         /* Save to be used when STOCK or ICON_SET property comes in */
         image->icon_size = g_value_get_int (value);
       break;
+    case PROP_PIXEL_SIZE:
+      gtk_image_set_pixel_size (image, g_value_get_int (value));
+      break;
     case PROP_PIXBUF_ANIMATION:
       gtk_image_set_from_animation (image,
                                     g_value_get_object (value));
       break;
-      
+    case PROP_ICON_NAME:
+      gtk_image_set_from_icon_name (image, g_value_get_string (value),
+				    image->icon_size);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -325,8 +371,10 @@ gtk_image_get_property (GObject     *object,
 			GParamSpec  *pspec)
 {
   GtkImage *image;
+  GtkImagePrivate *priv;
 
   image = GTK_IMAGE (object);
+  priv = GTK_IMAGE_GET_PRIVATE (image);
 
   /* The "getter" functions whine if you try to get the wrong
    * storage type. This function is instead robust against that,
@@ -360,6 +408,9 @@ gtk_image_get_property (GObject     *object,
         g_value_set_object (value,
                             image->data.image.image);
       break;
+    case PROP_FILE:
+      g_value_set_string (value, priv->filename);
+      break;
     case PROP_STOCK:
       if (image->storage_type != GTK_IMAGE_STOCK)
         g_value_set_string (value, NULL);
@@ -377,6 +428,9 @@ gtk_image_get_property (GObject     *object,
     case PROP_ICON_SIZE:
       g_value_set_int (value, image->icon_size);
       break;
+    case PROP_PIXEL_SIZE:
+      g_value_set_int (value, priv->pixel_size);
+      break;
     case PROP_PIXBUF_ANIMATION:
       if (image->storage_type != GTK_IMAGE_ANIMATION)
         g_value_set_object (value, NULL);
@@ -384,6 +438,13 @@ gtk_image_get_property (GObject     *object,
         g_value_set_object (value,
                             image->data.anim.anim);
       break;
+    case PROP_ICON_NAME:
+      if (image->storage_type != GTK_IMAGE_ICON_NAME)
+	g_value_set_string (value, NULL);
+      else
+	g_value_set_string (value,
+			    image->data.name.icon_name);
+      break;
     case PROP_STORAGE_TYPE:
       g_value_set_enum (value, image->storage_type);
       break;
@@ -401,7 +462,7 @@ gtk_image_get_property (GObject     *object,
  * @mask: a #GdkBitmap, or %NULL
  * 
  * Creates a #GtkImage widget displaying @pixmap with a @mask.
- * A #GdkImage is a server-side image buffer in the pixel format of the
+ * A #GdkPixmap is a server-side image buffer in the pixel format of the
  * current display. The #GtkImage does not assume a reference to the
  * pixmap or mask; you still need to unref them if you own references.
  * #GtkImage will add its own reference rather than adopting yours.
@@ -516,9 +577,9 @@ gtk_image_new_from_pixbuf (GdkPixbuf *pixbuf)
  * @size: a stock icon size
  * 
  * Creates a #GtkImage displaying a stock icon. Sample stock icon
- * names are #GTK_STOCK_OPEN, #GTK_STOCK_EXIT. Sample stock sizes
+ * names are #GTK_STOCK_OPEN, #GTK_STOCK_QUIT. Sample stock sizes
  * are #GTK_ICON_SIZE_MENU, #GTK_ICON_SIZE_SMALL_TOOLBAR. If the stock
- * icon name isn't known, a "broken image" icon will be displayed instead.
+ * icon name isn't known, the image will be empty.
  * You can register your own stock icon names, see
  * gtk_icon_factory_add_default() and gtk_icon_factory_add().
  * 
@@ -578,7 +639,12 @@ gtk_image_new_from_icon_set (GtkIconSet     *icon_set,
  * The #GtkImage does not assume a reference to the
  * animation; you still need to unref it if you own references.
  * #GtkImage will add its own reference rather than adopting yours.
- * 
+ *
+ * Note that the animation frames are shown using a timeout with
+ * #G_PRIORITY_DEFAULT. When using animations to indicate busyness,
+ * keep in mind that the animation will only be shown if the main loop
+ * is not busy with something that has a higher priority.
+ *
  * Return value: a new #GtkImage widget
  **/
 GtkWidget*
@@ -595,6 +661,33 @@ gtk_image_new_from_animation (GdkPixbufAnimation *animation)
   return GTK_WIDGET (image);
 }
 
+/**
+ * gtk_image_new_from_icon_name:
+ * @icon_name: an icon name
+ * @size: a stock icon size
+ * 
+ * Creates a #GtkImage displaying an icon from the current icon theme.
+ * If the icon name isn't known, a "broken image" icon will be
+ * displayed instead.  If the current icon theme is changed, the icon
+ * will be updated appropriately.
+ * 
+ * Return value: a new #GtkImage displaying the themed icon
+ *
+ * Since: 2.6
+ **/
+GtkWidget*
+gtk_image_new_from_icon_name (const gchar    *icon_name,
+			      GtkIconSize     size)
+{
+  GtkImage *image;
+
+  image = g_object_new (GTK_TYPE_IMAGE, NULL);
+
+  gtk_image_set_from_icon_name (image, icon_name, size);
+
+  return GTK_WIDGET (image);
+}
+
 /**
  * gtk_image_set_from_pixmap:
  * @image: a #GtkImage
@@ -623,7 +716,7 @@ gtk_image_set_from_pixmap (GtkImage  *image,
   if (mask)
     g_object_ref (mask);
 
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   image->mask = mask;
   
@@ -675,7 +768,7 @@ gtk_image_set_from_image  (GtkImage  *image,
   if (mask)
     g_object_ref (mask);
 
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (gdk_image)
     {
@@ -711,20 +804,22 @@ void
 gtk_image_set_from_file   (GtkImage    *image,
                            const gchar *filename)
 {
+  GtkImagePrivate *priv = GTK_IMAGE_GET_PRIVATE (image);
   GdkPixbufAnimation *anim;
   
   g_return_if_fail (GTK_IS_IMAGE (image));
 
   g_object_freeze_notify (G_OBJECT (image));
   
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (filename == NULL)
     {
+      priv->filename = NULL;
       g_object_thaw_notify (G_OBJECT (image));
       return;
     }
-  
+
   anim = gdk_pixbuf_animation_new_from_file (filename, NULL);
 
   if (anim == NULL)
@@ -742,17 +837,15 @@ gtk_image_set_from_file   (GtkImage    *image,
    */
 
   if (gdk_pixbuf_animation_is_static_image (anim))
-    {
-      gtk_image_set_from_pixbuf (image,
-                                 gdk_pixbuf_animation_get_static_image (anim));
-    }
+    gtk_image_set_from_pixbuf (image,
+			       gdk_pixbuf_animation_get_static_image (anim));
   else
-    {
-      gtk_image_set_from_animation (image, anim);
-    }
+    gtk_image_set_from_animation (image, anim);
 
   g_object_unref (anim);
 
+  priv->filename = g_strdup (filename);
+  
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -777,7 +870,7 @@ gtk_image_set_from_pixbuf (GtkImage  *image,
   if (pixbuf)
     g_object_ref (pixbuf);
 
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (pixbuf != NULL)
     {
@@ -818,7 +911,7 @@ gtk_image_set_from_stock  (GtkImage       *image,
   /* in case stock_id == image->data.stock.stock_id */
   new_id = g_strdup (stock_id);
   
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (new_id)
     {
@@ -834,7 +927,7 @@ gtk_image_set_from_stock  (GtkImage       *image,
     }
 
   g_object_notify (G_OBJECT (image), "stock");
-  g_object_notify (G_OBJECT (image), "icon_size");
+  g_object_notify (G_OBJECT (image), "icon-size");
   
   g_object_thaw_notify (G_OBJECT (image));
 }
@@ -860,7 +953,7 @@ gtk_image_set_from_icon_set  (GtkImage       *image,
   if (icon_set)
     gtk_icon_set_ref (icon_set);
   
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (icon_set)
     {      
@@ -874,8 +967,8 @@ gtk_image_set_from_icon_set  (GtkImage       *image,
        */
     }
   
-  g_object_notify (G_OBJECT (image), "icon_set");
-  g_object_notify (G_OBJECT (image), "icon_size");
+  g_object_notify (G_OBJECT (image), "icon-set");
+  g_object_notify (G_OBJECT (image), "icon-size");
   
   g_object_thaw_notify (G_OBJECT (image));
 }
@@ -901,7 +994,7 @@ gtk_image_set_from_animation (GtkImage           *image,
   if (animation)
     g_object_ref (animation);
 
-  gtk_image_reset (image);
+  gtk_image_clear (image);
 
   if (animation != NULL)
     {
@@ -916,7 +1009,52 @@ gtk_image_set_from_animation (GtkImage           *image,
                              gdk_pixbuf_animation_get_height (animation));
     }
 
-  g_object_notify (G_OBJECT (image), "pixbuf_animation");
+  g_object_notify (G_OBJECT (image), "pixbuf-animation");
+  
+  g_object_thaw_notify (G_OBJECT (image));
+}
+
+/**
+ * gtk_image_set_from_icon_name:
+ * @image: a #GtkImage
+ * @icon_name: an icon name
+ * @size: an icon size
+ *
+ * See gtk_image_new_from_icon_name() for details.
+ * 
+ * Since: 2.6
+ **/
+void
+gtk_image_set_from_icon_name  (GtkImage       *image,
+			       const gchar    *icon_name,
+			       GtkIconSize     size)
+{
+  gchar *new_name;
+  
+  g_return_if_fail (GTK_IS_IMAGE (image));
+
+  g_object_freeze_notify (G_OBJECT (image));
+
+  /* in case icon_name == image->data.name.icon_name */
+  new_name = g_strdup (icon_name);
+  
+  gtk_image_clear (image);
+
+  if (new_name)
+    {
+      image->storage_type = GTK_IMAGE_ICON_NAME;
+      
+      image->data.name.icon_name = new_name;
+      image->icon_size = size;
+
+      /* Size is demand-computed in size request method
+       * if we're a icon theme image, since changing the
+       * style impacts the size request
+       */
+    }
+
+  g_object_notify (G_OBJECT (image), "icon-name");
+  g_object_notify (G_OBJECT (image), "icon-size");
   
   g_object_thaw_notify (G_OBJECT (image));
 }
@@ -1108,6 +1246,39 @@ gtk_image_get_animation (GtkImage *image)
   return image->data.anim.anim;
 }
 
+/**
+ * gtk_image_get_icon_name:
+ * @image: a #GtkImage
+ * @icon_name: place to store an icon name
+ * @size: place to store an icon size
+ *
+ * Gets the icon name and size being displayed by the #GtkImage.
+ * The storage type of the image must be %GTK_IMAGE_EMPTY or
+ * %GTK_IMAGE_ICON_NAME (see gtk_image_get_storage_type()).
+ * The returned string is owned by the #GtkImage and should not
+ * be freed.
+ * 
+ * Since: 2.6
+ **/
+void
+gtk_image_get_icon_name  (GtkImage              *image,
+			  G_CONST_RETURN gchar **icon_name,
+			  GtkIconSize           *size)
+{
+  g_return_if_fail (GTK_IS_IMAGE (image));
+  g_return_if_fail (image->storage_type == GTK_IMAGE_ICON_NAME ||
+                    image->storage_type == GTK_IMAGE_EMPTY);
+
+  if (image->storage_type == GTK_IMAGE_EMPTY)
+    image->data.name.icon_name = NULL;
+  
+  if (icon_name)
+    *icon_name = image->data.name.icon_name;
+
+  if (size)
+    *size = image->icon_size;
+}
+
 /**
  * gtk_image_new:
  * 
@@ -1167,8 +1338,8 @@ gtk_image_unmap (GtkWidget *widget)
 {
   gtk_image_reset_anim_iter (GTK_IMAGE (widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->unmap)
-    GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->unmap)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->unmap (widget);
 }
 
 static void
@@ -1176,8 +1347,8 @@ gtk_image_unrealize (GtkWidget *widget)
 {
   gtk_image_reset_anim_iter (GTK_IMAGE (widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->unrealize)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->unrealize (widget);
 }
 
 static gint
@@ -1198,14 +1369,149 @@ animation_timeout (gpointer data)
       g_timeout_add (gdk_pixbuf_animation_iter_get_delay_time (image->data.anim.iter),
                      animation_timeout,
                      image);
-  
+
   gtk_widget_queue_draw (GTK_WIDGET (image));
 
+  if (GTK_WIDGET_DRAWABLE (image))
+    gdk_window_process_updates (GTK_WIDGET (image)->window, TRUE);
+
   GDK_THREADS_LEAVE ();
 
   return FALSE;
 }
 
+static void
+icon_theme_changed (GtkImage *image)
+{
+  if (image->storage_type == GTK_IMAGE_ICON_NAME) 
+    {
+      if (image->data.name.pixbuf)
+	g_object_unref (image->data.name.pixbuf);
+      image->data.name.pixbuf = NULL;
+
+      gtk_widget_queue_draw (GTK_WIDGET (image));
+    }
+}
+
+static void
+ensure_pixbuf_for_icon_name (GtkImage *image)
+{
+  GtkImagePrivate *priv;
+  GdkScreen *screen;
+  GtkIconTheme *icon_theme;
+  GtkSettings *settings;
+  gint width, height;
+  gint *sizes, *s, dist;
+  GError *error = NULL;
+
+  g_return_if_fail (image->storage_type == GTK_IMAGE_ICON_NAME);
+
+  priv = GTK_IMAGE_GET_PRIVATE (image);
+  screen = gtk_widget_get_screen (GTK_WIDGET (image));
+  icon_theme = gtk_icon_theme_get_for_screen (screen);
+  settings = gtk_settings_get_for_screen (screen);
+  if (image->data.name.pixbuf == NULL)
+    {
+      if (priv->pixel_size != -1)
+	{
+	  width = height = priv->pixel_size;
+	}
+      else if (!gtk_icon_size_lookup_for_settings (settings,
+						   image->icon_size,
+						   &width, &height))
+	{
+	  if (image->icon_size == -1)
+	    {
+	      /* Find an available size close to 48 */
+	      sizes = gtk_icon_theme_get_icon_sizes (icon_theme, image->data.name.icon_name);
+	      dist = 100;
+	      width = height = 48;
+	      for (s = sizes; *s; s++)
+		{
+		  if (*s == -1)
+		    {
+		      width = height = 48;
+		      break;
+		    }
+		  if (*s < 48)
+		    {
+		      if (48 - *s < dist)
+			{
+			  width = height = *s;
+			  dist = 48 - *s;
+			}
+		    }
+		  else
+		    {
+		      if (*s - 48 < dist)
+			{
+			  width = height = *s;
+			  dist = *s - 48;
+			}
+		    }
+		}
+	      g_free (sizes);
+	    }
+	  else
+	    {
+	      g_warning ("Invalid icon size %d\n", image->icon_size);
+	      width = height = 24;
+	    }
+	}
+      image->data.name.pixbuf =
+	gtk_icon_theme_load_icon (icon_theme,
+				  image->data.name.icon_name,
+				  MIN (width, height), 0, &error);
+      if (image->data.name.pixbuf == NULL)
+	{
+	  g_error_free (error);
+	  image->data.name.pixbuf =
+	    gtk_widget_render_icon (GTK_WIDGET (image),
+				    GTK_STOCK_MISSING_IMAGE,
+				    image->icon_size,
+				    NULL);
+	}
+    }
+}
+
+/*
+ * Like gdk_rectangle_intersect (dest, src, dest), but make 
+ * sure that the origin of dest is moved by an "even" offset. 
+ * If necessary grow the intersection by one row or column 
+ * to achieve this.
+ *
+ * This is necessary since we can't pass alignment information
+ * for the pixelation pattern down to gdk_pixbuf_saturate_and_pixelate(), 
+ * thus we have to makesure that the subimages are properly aligned.
+ */
+static gboolean
+rectangle_intersect_even (GdkRectangle *src, 
+			  GdkRectangle *dest)
+{
+  gboolean isect;
+  gint x, y;
+
+  x = dest->x;
+  y = dest->y;
+  isect = gdk_rectangle_intersect (dest, src, dest);
+
+  if ((dest->x - x + dest->y - y) % 2 != 0)
+    {
+      if (dest->x > x)
+	{
+	  dest->x--;
+	  dest->width++;
+	}
+      else
+	{
+	  dest->y--;
+	  dest->height++;
+	}
+    }
+  
+  return isect;
+}
+
 static gint
 gtk_image_expose (GtkWidget      *widget,
 		  GdkEventExpose *event)
@@ -1220,11 +1526,11 @@ gtk_image_expose (GtkWidget      *widget,
       GtkMisc *misc;
       GdkRectangle area, image_bound;
       gfloat xalign;
-      gint x, y;
+      gint x, y, mask_x, mask_y;
       GdkBitmap *mask;
       GdkPixbuf *pixbuf;
       gboolean needs_state_transform;
-      
+
       image = GTK_IMAGE (widget);
       misc = GTK_MISC (widget);
 
@@ -1247,11 +1553,11 @@ gtk_image_expose (GtkWidget      *widget,
 	xalign = 1.0 - misc->xalign;
   
       x = floor (widget->allocation.x + misc->xpad
-		 + ((widget->allocation.width - widget->requisition.width) * xalign)
-		 + 0.5);
+		 + ((widget->allocation.width - widget->requisition.width) * xalign));
       y = floor (widget->allocation.y + misc->ypad 
-		 + ((widget->allocation.height - widget->requisition.height) * misc->yalign)
-		 + 0.5);
+		 + ((widget->allocation.height - widget->requisition.height) * misc->yalign));
+      mask_x = x;
+      mask_y = y;
       
       image_bound.x = x;
       image_bound.y = y;      
@@ -1269,8 +1575,7 @@ gtk_image_expose (GtkWidget      *widget,
           gdk_drawable_get_size (image->data.pixmap.pixmap,
                                  &image_bound.width,
                                  &image_bound.height);
-
-	  if (gdk_rectangle_intersect (&image_bound, &area, &image_bound) &&
+	  if (rectangle_intersect_even (&area, &image_bound) &&
 	      needs_state_transform)
             {
               pixbuf = gdk_pixbuf_get_from_drawable (NULL,
@@ -1292,7 +1597,7 @@ gtk_image_expose (GtkWidget      *widget,
           image_bound.width = image->data.image.image->width;
           image_bound.height = image->data.image.image->height;
 
-	  if (gdk_rectangle_intersect (&image_bound, &area, &image_bound) &&
+	  if (rectangle_intersect_even (&area, &image_bound) &&
 	      needs_state_transform)
             {
               pixbuf = gdk_pixbuf_get_from_image (NULL,
@@ -1310,9 +1615,9 @@ gtk_image_expose (GtkWidget      *widget,
 
         case GTK_IMAGE_PIXBUF:
           image_bound.width = gdk_pixbuf_get_width (image->data.pixbuf.pixbuf);
-          image_bound.height = gdk_pixbuf_get_height (image->data.pixbuf.pixbuf);          
+          image_bound.height = gdk_pixbuf_get_height (image->data.pixbuf.pixbuf);            
 
-	  if (gdk_rectangle_intersect (&image_bound, &area, &image_bound) &&
+	  if (rectangle_intersect_even (&area, &image_bound) &&
 	      needs_state_transform)
 	    {
 	      pixbuf = gdk_pixbuf_new_subpixbuf (image->data.pixbuf.pixbuf,
@@ -1389,6 +1694,17 @@ gtk_image_expose (GtkWidget      *widget,
           }
           break;
 
+	case GTK_IMAGE_ICON_NAME:
+	  ensure_pixbuf_for_icon_name (image);
+	  pixbuf = image->data.name.pixbuf;
+	  if (pixbuf)
+	    {
+	      g_object_ref (pixbuf);
+	      image_bound.width = gdk_pixbuf_get_width (pixbuf);
+	      image_bound.height = gdk_pixbuf_get_height (pixbuf);
+	    }
+	  break;
+
         case GTK_IMAGE_EMPTY:
           g_assert_not_reached ();
           break;
@@ -1397,10 +1713,10 @@ gtk_image_expose (GtkWidget      *widget,
       if (mask)
 	{
 	  gdk_gc_set_clip_mask (widget->style->black_gc, mask);
-	  gdk_gc_set_clip_origin (widget->style->black_gc, x, y);
+	  gdk_gc_set_clip_origin (widget->style->black_gc, mask_x, mask_y);
 	}
 
-      if (gdk_rectangle_intersect (&image_bound, &area, &image_bound))
+      if (rectangle_intersect_even (&area, &image_bound))
         {
           if (pixbuf)
             {
@@ -1447,9 +1763,6 @@ gtk_image_expose (GtkWidget      *widget,
 				   image_bound.height,
 				   GDK_RGB_DITHER_NORMAL,
 				   0, 0);
-
-                  g_object_unref (pixbuf);
-                  pixbuf = NULL;
                 }
             }
           else
@@ -1478,6 +1791,7 @@ gtk_image_expose (GtkWidget      *widget,
                 case GTK_IMAGE_STOCK:
                 case GTK_IMAGE_ICON_SET:
                 case GTK_IMAGE_ANIMATION:
+		case GTK_IMAGE_ICON_NAME:
                 case GTK_IMAGE_EMPTY:
                   g_assert_not_reached ();
                   break;
@@ -1491,18 +1805,25 @@ gtk_image_expose (GtkWidget      *widget,
           gdk_gc_set_clip_origin (widget->style->black_gc, 0, 0);
         }
       
+      if (pixbuf)
+	g_object_unref (pixbuf);
+
     } /* if widget is drawable */
 
   return FALSE;
 }
 
 static void
-gtk_image_clear (GtkImage *image)
+gtk_image_reset (GtkImage *image)
 {
+  GtkImagePrivate *priv;
+
+  priv = GTK_IMAGE_GET_PRIVATE (image);
+
   g_object_freeze_notify (G_OBJECT (image));
   
   if (image->storage_type != GTK_IMAGE_EMPTY)
-    g_object_notify (G_OBJECT (image), "storage_type");
+    g_object_notify (G_OBJECT (image), "storage-type");
 
   if (image->mask)
     {
@@ -1514,7 +1835,7 @@ gtk_image_clear (GtkImage *image)
   if (image->icon_size != DEFAULT_ICON_SIZE)
     {
       image->icon_size = DEFAULT_ICON_SIZE;
-      g_object_notify (G_OBJECT (image), "icon_size");
+      g_object_notify (G_OBJECT (image), "icon-size");
     }
   
   switch (image->storage_type)
@@ -1562,22 +1883,31 @@ gtk_image_clear (GtkImage *image)
         gtk_icon_set_unref (image->data.icon_set.icon_set);
       image->data.icon_set.icon_set = NULL;
       
-      g_object_notify (G_OBJECT (image), "icon_set");      
+      g_object_notify (G_OBJECT (image), "icon-set");      
       break;
 
     case GTK_IMAGE_ANIMATION:
-      if (image->data.anim.frame_timeout)
-        g_source_remove (image->data.anim.frame_timeout);
+      gtk_image_reset_anim_iter (image);
       
       if (image->data.anim.anim)
         g_object_unref (image->data.anim.anim);
-
-      image->data.anim.frame_timeout = 0;
       image->data.anim.anim = NULL;
       
-      g_object_notify (G_OBJECT (image), "pixbuf_animation");
+      g_object_notify (G_OBJECT (image), "pixbuf-animation");
       
       break;
+
+    case GTK_IMAGE_ICON_NAME:
+      if (image->data.name.icon_name)
+	g_free (image->data.name.icon_name);
+      image->data.name.icon_name = NULL;
+      if (image->data.name.pixbuf)
+	g_object_unref (image->data.name.pixbuf);
+      image->data.name.pixbuf = NULL;
+
+      g_object_notify (G_OBJECT (image), "icon-name");
+
+      break;
       
     case GTK_IMAGE_EMPTY:
     default:
@@ -1585,6 +1915,13 @@ gtk_image_clear (GtkImage *image)
       
     }
 
+  if (priv->filename)
+    {
+      g_free (priv->filename);
+      priv->filename = NULL;
+      g_object_notify (G_OBJECT (image), "file");
+    }
+
   image->storage_type = GTK_IMAGE_EMPTY;
 
   memset (&image->data, '\0', sizeof (image->data));
@@ -1592,10 +1929,18 @@ gtk_image_clear (GtkImage *image)
   g_object_thaw_notify (G_OBJECT (image));
 }
 
-static void
-gtk_image_reset (GtkImage *image)
+/**
+ * gtk_image_clear:
+ * @image: a #GtkImage
+ *
+ * Resets the image to be empty.
+ *
+ * Since: 2.8
+ */
+void
+gtk_image_clear (GtkImage *image)
 {
-  gtk_image_clear (image);
+  gtk_image_reset (image);
 
   gtk_image_update_size (image, 0, 0);
 }
@@ -1615,7 +1960,7 @@ gtk_image_calc_size (GtkImage *image)
     {
     case GTK_IMAGE_STOCK:
       pixbuf = gtk_widget_render_icon (widget,
-                                       image->data.stock.stock_id,
+				       image->data.stock.stock_id,
                                        image->icon_size,
                                        NULL);
       break;
@@ -1629,7 +1974,11 @@ gtk_image_calc_size (GtkImage *image)
                                          widget,
                                          NULL);
       break;
-      
+    case GTK_IMAGE_ICON_NAME:
+      ensure_pixbuf_for_icon_name (image);
+      pixbuf = image->data.name.pixbuf;
+      if (pixbuf) g_object_ref (pixbuf);
+      break;
     default:
       break;
     }
@@ -1654,9 +2003,38 @@ gtk_image_size_request (GtkWidget      *widget,
   gtk_image_calc_size (image);
 
   /* Chain up to default that simply reads current requisition */
-  GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+  GTK_WIDGET_CLASS (gtk_image_parent_class)->size_request (widget, requisition);
 }
 
+static void
+gtk_image_style_set (GtkWidget      *widget,
+		     GtkStyle       *prev_style)
+{
+  GtkImage *image;
+
+  image = GTK_IMAGE (widget);
+
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->style_set)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->style_set (widget, prev_style);
+  
+  icon_theme_changed (image);
+}
+
+static void
+gtk_image_screen_changed (GtkWidget *widget,
+			  GdkScreen *prev_screen)
+{
+  GtkImage *image;
+
+  image = GTK_IMAGE (widget);
+
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->screen_changed)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->screen_changed (widget, prev_screen);
+
+  icon_theme_changed (image);
+}
+
+
 static void
 gtk_image_update_size (GtkImage *image,
                        gint      image_width,
@@ -1668,3 +2046,101 @@ gtk_image_update_size (GtkImage *image,
   if (GTK_WIDGET_VISIBLE (image))
     gtk_widget_queue_resize (GTK_WIDGET (image));
 }
+
+
+/**
+ * gtk_image_set_pixel_size:
+ * @image: a #GtkImage
+ * @pixel_size: the new pixel size
+ * 
+ * Sets the pixel size to use for named icons. If the pixel size is set
+ * to a value != -1, it is used instead of the icon size set by
+ * gtk_image_set_from_icon_name().
+ *
+ * Since: 2.6
+ */
+void 
+gtk_image_set_pixel_size (GtkImage *image,
+			  gint      pixel_size)
+{
+  GtkImagePrivate *priv;
+
+  g_return_if_fail (GTK_IS_IMAGE (image));
+  
+  priv = GTK_IMAGE_GET_PRIVATE (image);
+
+  if (priv->pixel_size != pixel_size)
+    {
+      priv->pixel_size = pixel_size;
+      
+      if (image->storage_type == GTK_IMAGE_ICON_NAME)
+	{
+	  if (image->data.name.pixbuf)
+	    {
+	      g_object_unref (image->data.name.pixbuf);
+	      image->data.name.pixbuf = NULL;
+	    }
+	  
+	  gtk_image_update_size (image, pixel_size, pixel_size);
+	}
+      
+      g_object_notify (G_OBJECT (image), "pixel-size");
+    }
+}
+
+/**
+ * gtk_image_get_pixel_size:
+ * @image: a #GtkImage
+ * 
+ * Gets the pixel size used for named icons.
+ *
+ * Returns: the pixel size used for named icons.
+ *
+ * Since: 2.6
+ */
+gint
+gtk_image_get_pixel_size (GtkImage *image)
+{
+  GtkImagePrivate *priv;
+
+  g_return_val_if_fail (GTK_IS_IMAGE (image), -1);
+  
+  priv = GTK_IMAGE_GET_PRIVATE (image);
+
+  return priv->pixel_size;
+}
+
+#ifdef G_OS_WIN32
+
+#undef gtk_image_new_from_file
+
+GtkWidget*
+gtk_image_new_from_file   (const gchar *filename)
+{
+  gchar *utf8_filename = g_locale_to_utf8 (filename, -1, NULL, NULL, NULL);
+  GtkWidget *retval;
+
+  retval = gtk_image_new_from_file_utf8 (utf8_filename);
+
+  g_free (utf8_filename);
+
+  return retval;
+}
+
+#undef gtk_image_set_from_file
+
+void
+gtk_image_set_from_file   (GtkImage    *image,
+                           const gchar *filename)
+{
+  gchar *utf8_filename = g_locale_to_utf8 (filename, -1, NULL, NULL, NULL);
+
+  gtk_image_set_from_file_utf8 (image, utf8_filename);
+
+  g_free (utf8_filename);
+}
+
+#endif
+
+#define __GTK_IMAGE_C__
+#include "gtkaliasdef.c"
