@@ -614,7 +614,7 @@ build_recent_items_list (GtkRecentManager *manager)
  * monitors the recently used resources list, and emits the "changed" signal
  * each time something inside the list changes.
  *
- * #GtkRecentManager objects are expansive: be sure to create them only when
+ * #GtkRecentManager objects are expensive: be sure to create them only when
  * needed. You should use the gtk_recent_manager_new_for_screen() or the
  * gtk_recent_manager_get_default() functions instead.
  *
@@ -1322,6 +1322,9 @@ gtk_recent_manager_get_items (GtkRecentManager *manager)
   priv = manager->priv;
   if (!priv->recent_items)
     return NULL;
+
+  if (priv->limit == 0)
+    return NULL;
   
   uris = g_bookmark_file_get_uris (priv->recent_items, &uris_len);
   for (i = 0; i < uris_len; i++)
@@ -2222,7 +2225,7 @@ get_uri_shortname_for_display (const gchar *uri)
       
       local_file = g_filename_from_uri (uri, NULL, NULL);
       
-      if (local_file != NULL)
+      if (local_file)
         {
           name = g_filename_display_basename (local_file);
           validated = TRUE;
@@ -2230,7 +2233,8 @@ get_uri_shortname_for_display (const gchar *uri)
 		
       g_free (local_file);
     } 
-  else
+  
+  if (!name)
     {
       gchar *method;
       gchar *local_file;
