@@ -24,8 +24,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtktable.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum
 {
@@ -51,8 +54,6 @@ enum
 };
   
 
-static void gtk_table_class_init    (GtkTableClass  *klass);
-static void gtk_table_init	    (GtkTable	    *table);
 static void gtk_table_finalize	    (GObject	    *object);
 static void gtk_table_size_request  (GtkWidget	    *widget,
 				     GtkRequisition *requisition);
@@ -97,35 +98,7 @@ static void gtk_table_size_allocate_pass1 (GtkTable *table);
 static void gtk_table_size_allocate_pass2 (GtkTable *table);
 
 
-static GtkContainerClass *parent_class = NULL;
-
-
-GType
-gtk_table_get_type (void)
-{
-  static GType table_type = 0;
-  
-  if (!table_type)
-    {
-      static const GTypeInfo table_info =
-      {
-	sizeof (GtkTableClass),
-	NULL,
-	NULL,
-	(GClassInitFunc) gtk_table_class_init,
-	NULL,
-	NULL,
-	sizeof (GtkTable),
-	0,
-	(GInstanceInitFunc) gtk_table_init,
-      };
-      
-      table_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkTable",
-					   &table_info, 0);
-    }
-  
-  return table_type;
-}
+G_DEFINE_TYPE (GtkTable, gtk_table, GTK_TYPE_CONTAINER)
 
 static void
 gtk_table_class_init (GtkTableClass *class)
@@ -134,8 +107,6 @@ gtk_table_class_init (GtkTableClass *class)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (class);
   
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->finalize = gtk_table_finalize;
 
   gobject_class->get_property = gtk_table_get_property;
@@ -154,104 +125,104 @@ gtk_table_class_init (GtkTableClass *class)
 
   g_object_class_install_property (gobject_class,
                                    PROP_N_ROWS,
-                                   g_param_spec_uint ("n_rows",
+                                   g_param_spec_uint ("n-rows",
 						     P_("Rows"),
 						     P_("The number of rows in the table"),
 						     0,
 						     G_MAXUINT,
 						     0,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_N_COLUMNS,
-                                   g_param_spec_uint ("n_columns",
+                                   g_param_spec_uint ("n-columns",
 						     P_("Columns"),
 						     P_("The number of columns in the table"),
 						     0,
 						     G_MAXUINT,
 						     0,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_ROW_SPACING,
-                                   g_param_spec_uint ("row_spacing",
+                                   g_param_spec_uint ("row-spacing",
 						     P_("Row spacing"),
 						     P_("The amount of space between two consecutive rows"),
 						     0,
 						     G_MAXUINT,
 						     0,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_COLUMN_SPACING,
-                                   g_param_spec_uint ("column_spacing",
+                                   g_param_spec_uint ("column-spacing",
 						     P_("Column spacing"),
 						     P_("The amount of space between two consecutive columns"),
 						     0,
 						     G_MAXUINT,
 						     0,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_HOMOGENEOUS,
                                    g_param_spec_boolean ("homogeneous",
-							 P_("Homogenous"),
-							 P_("If TRUE this means the table cells are all the same width/height"),
+							 P_("Homogeneous"),
+							 P_("If TRUE, the table cells are all the same width/height"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_LEFT_ATTACH,
-					      g_param_spec_uint ("left_attach", 
+					      g_param_spec_uint ("left-attach", 
 								 P_("Left attachment"), 
 								 P_("The column number to attach the left side of the child to"),
 								 0, 65535, 0,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_RIGHT_ATTACH,
-					      g_param_spec_uint ("right_attach", 
+					      g_param_spec_uint ("right-attach", 
 								 P_("Right attachment"), 
 								 P_("The column number to attach the right side of a child widget to"),
 								 1, 65535, 1,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_TOP_ATTACH,
-					      g_param_spec_uint ("top_attach", 
+					      g_param_spec_uint ("top-attach", 
 								 P_("Top attachment"), 
 								 P_("The row number to attach the top of a child widget to"),
 								 0, 65535, 0,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_BOTTOM_ATTACH,
-					      g_param_spec_uint ("bottom_attach",
+					      g_param_spec_uint ("bottom-attach",
 								 P_("Bottom attachment"), 
 								 P_("The row number to attach the bottom of the child to"),
 								 1, 65535, 1,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_X_OPTIONS,
-					      g_param_spec_flags ("x_options", 
+					      g_param_spec_flags ("x-options", 
 								  P_("Horizontal options"), 
 								  P_("Options specifying the horizontal behaviour of the child"),
 								  GTK_TYPE_ATTACH_OPTIONS, GTK_EXPAND | GTK_FILL,
-								  G_PARAM_READWRITE));
+								  GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y_OPTIONS,
-					      g_param_spec_flags ("y_options", 
+					      g_param_spec_flags ("y-options", 
 								  P_("Vertical options"), 
 								  P_("Options specifying the vertical behaviour of the child"),
 								  GTK_TYPE_ATTACH_OPTIONS, GTK_EXPAND | GTK_FILL,
-								  G_PARAM_READWRITE));
+								  GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_X_PADDING,
-					      g_param_spec_uint ("x_padding", 
+					      g_param_spec_uint ("x-padding", 
 								 P_("Horizontal padding"), 
 								 P_("Extra space to put between the child and its left and right neighbors, in pixels"),
 								 0, 65535, 0,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y_PADDING,
-					      g_param_spec_uint ("y_padding", 
+					      g_param_spec_uint ("y-padding", 
 								 P_("Vertical padding"), 
 								 P_("Extra space to put between the child and its upper and lower neighbors, in pixels"),
 								 0, 65535, 0,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
 }
 
 static GType
@@ -551,7 +522,7 @@ gtk_table_resize (GtkTable *table,
 	      table->rows[i].shrink = 0;
 	    }
 
-	  g_object_notify (G_OBJECT (table), "n_rows");
+	  g_object_notify (G_OBJECT (table), "n-rows");
 	}
 
       if (n_cols != table->ncols)
@@ -573,7 +544,7 @@ gtk_table_resize (GtkTable *table,
 	      table->cols[i].shrink = 0;
 	    }
 
-	  g_object_notify (G_OBJECT (table), "n_columns");
+	  g_object_notify (G_OBJECT (table), "n-columns");
 	}
     }
 }
@@ -732,7 +703,7 @@ gtk_table_set_row_spacings (GtkTable *table,
   if (GTK_WIDGET_VISIBLE (table))
     gtk_widget_queue_resize (GTK_WIDGET (table));
 
-  g_object_notify (G_OBJECT (table), "row_spacing");
+  g_object_notify (G_OBJECT (table), "row-spacing");
 }
 
 /**
@@ -768,7 +739,7 @@ gtk_table_set_col_spacings (GtkTable *table,
   if (GTK_WIDGET_VISIBLE (table))
     gtk_widget_queue_resize (GTK_WIDGET (table));
 
-  g_object_notify (G_OBJECT (table), "column_spacing");
+  g_object_notify (G_OBJECT (table), "column-spacing");
 }
 
 /**
@@ -836,7 +807,7 @@ gtk_table_finalize (GObject *object)
   g_free (table->rows);
   g_free (table->cols);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_table_parent_class)->finalize (object);
 }
 
 static void
@@ -1664,3 +1635,6 @@ gtk_table_size_allocate_pass2 (GtkTable *table)
 	}
     }
 }
+
+#define __GTK_TABLE_C__
+#include "gtkaliasdef.c"
