@@ -41,6 +41,19 @@ G_BEGIN_DECLS
  * GtkTextBTree is the PRIVATE internal representation of it.
  */
 
+/* these values are used as "info" for the targets contained in the
+ * lists returned by gtk_text_buffer_get_copy,paste_target_list()
+ *
+ * the enum counts down from G_MAXUINT to avoid clashes with application
+ * added drag destinations which usually start at 0.
+ */
+typedef enum
+{
+  GTK_TEXT_BUFFER_TARGET_INFO_BUFFER_CONTENTS = G_MAXUINT - 0,
+  GTK_TEXT_BUFFER_TARGET_INFO_RICH_TEXT       = G_MAXUINT - 1,
+  GTK_TEXT_BUFFER_TARGET_INFO_TEXT            = G_MAXUINT - 2
+} GtkTextBufferTargetInfo;
+
 typedef struct _GtkTextBTree GtkTextBTree;
 
 typedef struct _GtkTextLogAttrCache GtkTextLogAttrCache;
@@ -70,6 +83,8 @@ struct _GtkTextBuffer
   
   /* Whether the buffer has been modified since last save */
   guint modified : 1;
+
+  guint has_selection : 1;
 };
 
 struct _GtkTextBufferClass
@@ -290,7 +305,7 @@ void gtk_text_buffer_remove_all_tags       (GtkTextBuffer     *buffer,
 GtkTextTag    *gtk_text_buffer_create_tag (GtkTextBuffer *buffer,
                                            const gchar   *tag_name,
                                            const gchar   *first_property_name,
-                                           ...) G_GNUC_NULL_TERMINATED;
+                                           ...);
 
 /* Obtain iterators pointed at various places, then you can move the
  * iterator around using the GtkTextIter operators
@@ -337,6 +352,8 @@ gboolean        gtk_text_buffer_get_modified            (GtkTextBuffer *buffer);
 void            gtk_text_buffer_set_modified            (GtkTextBuffer *buffer,
                                                          gboolean       setting);
 
+gboolean        gtk_text_buffer_get_has_selection       (GtkTextBuffer *buffer);
+
 void gtk_text_buffer_add_selection_clipboard    (GtkTextBuffer     *buffer,
 						 GtkClipboard      *clipboard);
 void gtk_text_buffer_remove_selection_clipboard (GtkTextBuffer     *buffer,
@@ -363,6 +380,9 @@ gboolean        gtk_text_buffer_delete_selection        (GtkTextBuffer *buffer,
 void            gtk_text_buffer_begin_user_action       (GtkTextBuffer *buffer);
 void            gtk_text_buffer_end_user_action         (GtkTextBuffer *buffer);
 
+GtkTargetList * gtk_text_buffer_get_copy_target_list    (GtkTextBuffer *buffer);
+GtkTargetList * gtk_text_buffer_get_paste_target_list   (GtkTextBuffer *buffer);
+
 /* INTERNAL private stuff */
 void            _gtk_text_buffer_spew                  (GtkTextBuffer      *buffer);
 
