@@ -37,11 +37,13 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
+#include <glib/gi18n.h>
 
 static gboolean force_update = FALSE;
 static gboolean ignore_theme_index = FALSE;
 static gboolean quiet = FALSE;
 static gboolean index_only = FALSE;
+static gchar *var_name = "-";
 
 #define CACHE_NAME "icon-theme.cache"
 
@@ -408,7 +410,7 @@ maybe_cache_image_data (Image       *image,
 	  idata2 = g_hash_table_lookup (image_data_hash, path2);
 
 	  if (idata && idata2 && idata != idata2)
-	    g_error ("different idatas found for symlinked '%s' and '%s'\n",
+	    g_error (_("different idatas found for symlinked '%s' and '%s'\n"),
 		     path, path2);
 
 	  if (idata && !idata2)
@@ -1111,19 +1113,19 @@ write_file (FILE *cache, GHashTable *files, GList *directories)
    * back and change it later */
   if (!write_header (cache, 0))
     {
-      g_printerr ("Failed to write header\n");
+      g_printerr (_("Failed to write header\n"));
       return FALSE;
     }
 
   if (!write_hash_table (cache, &context, &new_offset))
     {
-      g_printerr ("Failed to write hash table\n");
+      g_printerr (_("Failed to write hash table\n"));
       return FALSE;
     }
 
   if (!write_dir_index (cache, new_offset, directories))
     {
-      g_printerr ("Failed to write directory index\n");
+      g_printerr (_("Failed to write directory index\n"));
       return FALSE;
     }
   
@@ -1131,7 +1133,7 @@ write_file (FILE *cache, GHashTable *files, GList *directories)
 
   if (!write_header (cache, new_offset))
     {
-      g_printerr ("Failed to rewrite header\n");
+      g_printerr (_("Failed to rewrite header\n"));
       return FALSE;
     }
     
@@ -1142,6 +1144,9 @@ void
 build_cache (const gchar *path)
 {
   gchar *cache_path, *tmp_cache_path;
+#ifdef G_OS_WIN32
+  gchar *bak_cache_path = NULL;
+#endif
   GHashTable *files;
   gboolean retval;
   FILE *cache;
@@ -1154,7 +1159,7 @@ build_cache (const gchar *path)
   
   if (!cache)
     {
-      g_printerr ("Failed to write cache file: %s\n", g_strerror (errno));
+      g_printerr (_("Failed to write cache file: %s\n"), g_strerror (errno));
       exit (1);
     }
 
@@ -1187,11 +1192,42 @@ build_cache (const gchar *path)
 
   cache_path = g_build_filename (path, CACHE_NAME, NULL);
 
+#ifdef G_OS_WIN32
+  if (g_file_test (cache_path, G_FILE_TEST_EXISTS))
+    {
+      bak_cache_path = g_strconcat (cache_path, ".bak", NULL);
+      g_unlink (bak_cache_path);
+      if (g_rename (cache_path, bak_cache_path) == -1)
+	{
+	  g_printerr (_("Could not rename %s to %s: %s, removing %s then.\n"),
+		      cache_path, bak_cache_path,
+		      g_strerror (errno),
+		      cache_path);
+	  g_unlink (cache_path);
+	  bak_cache_path = NULL;
+	}
+    }
+#endif
+
   if (g_rename (tmp_cache_path, cache_path) == -1)
     {
+      g_printerr (_("Could not rename %s to %s: %s\n"),
+		  tmp_cache_path, cache_path,
+		  g_strerror (errno));
       g_unlink (tmp_cache_path);
+#ifdef G_OS_WIN32
+      if (bak_cache_path != NULL)
+	if (g_rename (bak_cache_path, cache_path) == -1)
+	  g_printerr (_("Could not rename %s back to %s: %s.\n"),
+		      bak_cache_path, cache_path,
+		      g_strerror (errno));
+#endif
       exit (1);
     }
+#ifdef G_OS_WIN32
+  if (bak_cache_path != NULL)
+    g_unlink (bak_cache_path);
+#endif
 
   /* Update time */
   /* FIXME: What do do if an error occurs here? */
@@ -1204,14 +1240,50 @@ build_cache (const gchar *path)
   utime (path, &utime_buf);
   
   if (!quiet)
-    g_printerr ("Cache file created successfully.\n");
+    g_printerr (_("Cache file created successfully.\n"));
+}
+
+void
+write_csource (const gchar *path)
+{
+  gchar *cache_path;
+  gchar *data;
+  gsize len;
+  gint i;
+
+  cache_path = g_build_filename (path, CACHE_NAME, NULL);
+  if (!g_file_get_contents (cache_path, &data, &len, NULL))
+    exit (1);
+  
+  g_printf ("#ifdef __SUNPRO_C\n");
+  g_printf ("#pragma align 4 (%s)\n", var_name);   
+  g_printf ("#endif\n");
+  
+  g_printf ("#ifdef __GNUC__\n");
+  g_printf ("static const guint8 %s[] __attribute__ ((__aligned__ (4))) = \n", var_name);
+  g_printf ("#else\n");
+  g_printf ("static const guint8 %s[] = \n", var_name);
+  g_printf ("#endif\n");
+
+  g_printf ("{\n");
+  for (i = 0; i < len - 1; i++)
+    {
+      if (i %12 == 0)
+	g_printf ("  ");
+      g_printf ("0x%02x, ", (guint8)data[i]);
+      if (i % 12 == 11)
+        g_printf ("\n");
+    }
+  
+  g_printf ("0x%02x\n};\n", (guint8)data[i]);
 }
 
 static GOptionEntry args[] = {
-  { "force", 'f', 0, G_OPTION_ARG_NONE, &force_update, "Overwrite an existing cache, even if uptodate", NULL },
-  { "ignore-theme-index", 't', 0, G_OPTION_ARG_NONE, &ignore_theme_index, "Don't check for the existence of index.theme", NULL },
-  { "index-only", 'i', 0, G_OPTION_ARG_NONE, &index_only, "Don't include image data in the cache", NULL },
-  { "quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet, "Turn off verbose output", NULL },
+  { "force", 'f', 0, G_OPTION_ARG_NONE, &force_update, N_("Overwrite an existing cache, even if uptodate"), NULL },
+  { "ignore-theme-index", 't', 0, G_OPTION_ARG_NONE, &ignore_theme_index, N_("Don't check for the existence of index.theme"), NULL },
+  { "index-only", 'i', 0, G_OPTION_ARG_NONE, &index_only, N_("Don't include image data in the cache"), NULL },
+  { "source", 'c', 0, G_OPTION_ARG_STRING, &var_name, N_("Output a C header file"), "NAME" },
+  { "quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet, N_("Turn off verbose output"), NULL },
   { NULL }
 };
 
@@ -1223,9 +1295,12 @@ main (int argc, char **argv)
 
   if (argc < 2)
     return 0;
+  
+  bindtextdomain (GETTEXT_PACKAGE, GTK_LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
   context = g_option_context_new ("ICONPATH");
-  g_option_context_add_main_entries (context, args, NULL);
+  g_option_context_add_main_entries (context, args, GETTEXT_PACKAGE);
 
   g_option_context_parse (context, &argc, &argv, NULL);
   
@@ -1236,8 +1311,8 @@ main (int argc, char **argv)
   
   if (!ignore_theme_index && !has_theme_index (path))
     {
-      g_printerr ("No theme index file in '%s'.\n"
-		  "If you really want to create an icon cache here, use --ignore-theme-index.\n", path);
+      g_printerr (_("No theme index file in '%s'.\n"
+		    "If you really want to create an icon cache here, use --ignore-theme-index.\n"), path);
       return 1;
     }
   
@@ -1247,5 +1322,8 @@ main (int argc, char **argv)
   g_type_init ();
   build_cache (path);
 
+  if (strcmp (var_name, "-") != 0)
+    write_csource (path);
+
   return 0;
 }
