@@ -23,10 +23,7 @@
 #include <gtk/gtkimcontext.h>
 #include "x11/gdkx.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 extern GType gtk_type_im_context_xim;
 
@@ -41,42 +38,6 @@ extern GType gtk_type_im_context_xim;
 typedef struct _GtkIMContextXIM       GtkIMContextXIM;
 typedef struct _GtkIMContextXIMClass  GtkIMContextXIMClass;
 
-typedef struct _GtkXIMInfo GtkXIMInfo;
-
-struct _GtkIMContextXIM
-{
-  GtkIMContext object;
-
-  GtkXIMInfo *im_info;
-
-  gchar *locale;
-  gchar *mb_charset;
-
-  GdkWindow *client_window;
-
-  gint preedit_size;
-  gint preedit_length;
-  gunichar *preedit_chars;
-  XIMFeedback *feedbacks;
-
-  gint preedit_cursor;
-  
-  XIMCallback preedit_start_callback;
-  XIMCallback preedit_done_callback;
-  XIMCallback preedit_draw_callback;
-  XIMCallback preedit_caret_callback;
-
-  XIMCallback status_start_callback;
-  XIMCallback status_done_callback;
-  XIMCallback status_draw_callback;
-
-  XIC ic;
-  gboolean filter_key_release;
-
-  guint use_preedit : 1;
-  guint status_visible : 1;
-};
-
 struct _GtkIMContextXIMClass
 {
   GtkIMContextClass parent_class;
@@ -87,9 +48,7 @@ GtkIMContext *gtk_im_context_xim_new (void);
 
 void gtk_im_context_xim_shutdown (void);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 
 #endif /* __GTK_IM_CONTEXT_XIM_H__ */
